function CongruenceTest(subjectName, trials, displayTime)
%Congruence Test: a speeded color-grapheme congruency task
%Presents a grapheme surrounded by a color for 1 second, with a
%50% chance that the color is incongruent with the subject's synesthetic percept.
%The subject answers cong/incong after each trial; reaction time is
%also recorded.
%
%This program is part of the TexSyn Toolbox developed by David Eagleman,PhD
%eagleman@synesthete.org
%Full package freely downloadable from www.synesthete.org

%set the present working directory here:
thisfcn = 'CongruenceTest';
rootdir = setrootdir(thisfcn);

%retrieve user input
if nargin<1
    prompt = {'Subject name:','Number of trials:','How long to display the grapheme (sec):'};
    dlg_title = 'TexSyn Toolbox';
    num_lines = 1;
    def = {'','108','1'};
    parameters = inputdlg(prompt,dlg_title,num_lines,def);
    pause(0.5);
    subjectName = parameters{1};
    numTrials = str2num(parameters{2});
    displayTime = str2num(parameters{3});
end

%first, load the subject's colors
myfilename = [rootdir 'Data\GraphemeColorResults_' subjectName '.mat']
load(myfilename)
row = 2; %take the second row from the 3 rows of colored alphabets
data=sortrows(data);
for i=1:3:108
    t=(i+2)/3;
    if sum(data(i+row-1,2:4))==-3
        data(i+row-1,2:4)=0;
    end
    colormat(1,t,1:3)=data(i+row-1,2:4);
end
colormat(1,37,1:3)=[.5 .5 .5];
colormat;

%open a figure window
myfig = figure;
set(gcf, 'position', [50 50 1200 800]);
w=25;
h=12.5;
% axis([0 w 0 h]);

validAnswerWindow = 2; %subjects have 2 seconds to register an answer or else they are reprimanded
theGraphemes = char([48:57 65:90]); %this makes the char array 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ
vec=Shuffle([theGraphemes theGraphemes theGraphemes]);  %shuffle up the presentation
size(vec)

for t=1:numTrials
    set(gcf, 'position', [50 50 1200 800]);

    graphemeColorRow = find(theGraphemes==vec(t));  %convert the grapheme for this trial to a row number
    
    incongruent = sample([0 1]);
 
    if incongruent      %draw a grapheme with an INCONGRUENT surround
        DiffColor=0; 
        ctr = 0;
        while DiffColor<.5 && ctr<50 %this ensures that an incogruent color is different by at least a certain amount
            surroundColor = ceil(rand*36)  %pick a random row
            r=abs(colormat(1,graphemeColorRow,1)-colormat(1,surroundColor,1));
            g=abs(colormat(1,graphemeColorRow,2)-colormat(1,surroundColor,2));
            b=abs(colormat(1,graphemeColorRow,3)-colormat(1,surroundColor,3));
            DiffColor=r+g+b;
            disp('*');
            ctr=ctr+1;
        end;   %while
        if ctr>49
            disp('ERROR: all the colors in this file are too similar. Exiting');
            exit();
        end
    else
            surroundColor = graphemeColorRow;
    end

    axis([0 w 0 h]);
    hold on
    %draw the surround
    image(.5*w-1,.5*h-1, colormat(1,surroundColor,1:3));
    image(.5*w,.5*h-1, colormat(1,surroundColor,1:3));
    image(.5*w+1,.5*h-1, colormat(1,surroundColor,1:3));
    image(.5*w+2,.5*h-1, colormat(1,surroundColor,1:3));
    image(.5*w-1,.5*h, colormat(1,surroundColor,1:3));
    image(.5*w+2,.5*h, colormat(1,surroundColor,1:3));
    image(.5*w-1,.5*h+1, colormat(1,surroundColor,1:3));
    image(.5*w,.5*h+1, colormat(1,surroundColor,1:3));
    image(.5*w+1,.5*h+1, colormat(1,surroundColor,1:3));
    image(.5*w+2,.5*h+1, colormat(1,surroundColor,1:3));
    image(.5*w,.5*h, colormat(1,37,1:3));
    image(.5*w+1,.5*h, colormat(1,37,1:3));
    %draw the grapheme in the middle
    text(.5*w-.5,.5*h,vec(t),'Fontsize', 40);
    axis off;

    pause(displayTime);  %after a pause, clear the screen
    clf;
    drawnow;
    
    %Now collect subject input
    starttime = GetSecs;
    tic
    button = askquestion('Was the letter congruent with the color?','Synesthesia','YES','NO','none');
    
    %give a warning message if they've taken too long
    if toc > validAnswerWindow
        close;
        figure(myfig);
        set(gcf, 'position', [50 50 1200 800]);
        clf;
        axis([0 w 0 h]);
        hold on
        text(.1*w-.5,.2*h,'Please respond more quickly.','Fontsize', 30);
        axis off;
        drawnow;
        pause(1);
        clf;
    end

    switch button
        case 'YES'
            if incongruent
                answer(t,1)=0;  %wrong answer
            else
                answer(t,1)=1;  %right answer
            end
        case 'NO'
            if incongruent
                answer(t,1)=1;  %right answer
            else
                answer(t,1)=0;  %wrong answer
            end
        otherwise
            %error in program
            answer(t,1) = -1;
    end; %switch

    RT = GetSecs - starttime;  %reaction time

    answer(t,2)=incongruent;
    answer(t,3)= RT;
    answer(t,4)=double(vec(t));
    answer(t,5:7) = colormat(1,graphemeColorRow,1:3);
    answer(t,8) = theGraphemes(surroundColor); 
    answer(t,9:11) = colormat(1,surroundColor,1:3);
 
    pause(.5*rand);
end

text(.1,.5,'Thank you for your participation', 'Fontsize',35)
axis off;
drawnow;

myfilename = getSafeFilename(thisfcn,subjectName);   %this saves the file in the Data directory, making sure not to overwrite
save(myfilename, 'answer');

fprintf('Score: %.2f percent\n',mean(answer(:,1)*100));
fprintf('Average Reaction Time: %.2f +- %.2f sec\n',mean(answer(:,3)),std(answer(:,3))); 

clf;
subplot(1,2,1); bar(mean(answer(:,1)*100)); axis([.5 1.5 0 100]); ylabel('% Correct');
subplot(1,2,2); bar(mean(answer(:,3))); ylabel('Reaction Time (sec)');